﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances
{
    public static class InvoiceExtension
    {
        /// <summary>
        /// 获取子发票的流水号
        /// </summary>
        /// <param name="invoice"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static string GetSubSerialNumber(this Invoice invoice, IDbConnection conn = null)
        {
            invoice.RootKvid.ThrowIfEmpty("RootKvid不能为空");
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<Invoice>();
            var query = conn.From<Invoice>();
            query.Where(o => o.Kvid == invoice.RootKvid);
            query.Select(o => new { o.SerialNumber, o.Kvid });
            var rootInvoice = conn.Single(query);
            rootInvoice.ThrowIfNull("主发票记录不存在");
            //allcount不需要加1，因为Root的也统计进去了
            var allcount = conn.Count<Invoice>(o => o.RootKvid == rootInvoice.Kvid);
            var subSerialNumber = $"{rootInvoice.SerialNumber}-{allcount}";
            var rtns = subSerialNumber;
            if (conn.Exists<Invoice>(o => o.SerialNumber == subSerialNumber))
            {
                rtns = $"{subSerialNumber}.{Extension.GetRandomCharsThreadSafe()}";
            }
            return rtns;
        }

        /// <summary>
        /// Payment与Invoice产生关联的方法
        /// </summary>
        /// <param name="invoices"></param>
        /// <param name="payments"></param>
        /// <returns></returns>
        public static List<Invoice> Correlating(this List<Invoice> invoices, List<Payment> payments)
        {
            payments.ThrowIfNullOrEmpty("要关联的Payments不能为空!");
            invoices.ThrowIfNullOrEmpty("要关联的Invoices不能为空!");
            (payments.Exists(o => o.Type != PaymentType.Bank && o.Type != PaymentType.AliPay && o.Type != PaymentType.Cash && o.Type != PaymentType.Pos && o.Type != PaymentType.Split && o.Type != PaymentType.WeChat)).ThrowIfTrue("请选择正确收款进行关联");
            var payedTime = payments.Max(o => o.OperateTime);

            var rtns = new List<Invoice>();

            foreach (var payment in payments)
            {
                //说明已经建立关联关系的payment 跳过
                if (payment.AmountInvoice >= payment.Amount) continue;
                foreach (var invoice in invoices)
                {
                    var amountLeave = payment.Amount - payment.AmountInvoice;//可以开票的金额
                    if (amountLeave <= 0) break;
                    var amountCorrelating = invoice.Amount - invoice.AmountPayment;//要关联的金额
                    if (amountCorrelating <= 0) continue;
                    var amount = amountLeave <= amountCorrelating ? amountLeave : amountCorrelating;

                    invoice.PayedTime = payedTime;
                    invoice.AddOnlyProperties(o => o.PayedTime);
                    invoice.AmountPayment += amount;
                    invoice.AddOnlyProperties(o => o.AmountPayment);

                    #region 建立关联关系
                    var relationInvoice = new Invoice();
                    relationInvoice.PopulateInstance(invoice);
                    relationInvoice.SerialNumber = string.Empty;
                    relationInvoice.Kvid = Guid.NewGuid();
                    relationInvoice.ParentKvid = invoice.Kvid;

                    relationInvoice.BizId = payment.SerialNumber;
                    relationInvoice.BizKvid = payment.RootKvid;//记录根的Kvid
                    relationInvoice.BizType = typeof(Payment).FullName;

                    relationInvoice.Type = "Relation";

                    relationInvoice.PayedTime = payedTime;
                    relationInvoice.Amount = amount;
                    relationInvoice.AmountPayment = amount;
                    //不含税金额
                    relationInvoice.AmountUntaxed = Math.Round(relationInvoice.Amount / (1 + relationInvoice.TaxRate), 2);
                    //税额
                    relationInvoice.AmountTax = relationInvoice.Amount - relationInvoice.AmountUntaxed;
                    relationInvoice.Summary = payment.PayerName;
                    rtns.Add(relationInvoice);
                    #endregion

                    payment.AmountInvoice += amount;
                    if (payment.InvoiceTime == null)
                    {
                        payment.InvoiceTime = invoice.OperateTime;
                        payment.AddOnlyProperties(o => o.InvoiceTime);
                    }
                    else
                    {
                        if (payment.InvoiceTime.Value < invoice.OperateTime)
                        {
                            payment.InvoiceTime = invoice.OperateTime;
                            payment.AddOnlyProperties(o => o.InvoiceTime);
                        }
                    }
                }
                if (payment.Metadata == null) payment.Metadata = new Dictionary<string, string>();
                payment.AddOnlyProperties(o => o.AmountInvoice);
                if (payment.Metadata.IsNullOrEmpty()) payment.Metadata = new Dictionary<string, string>();
                payment.Metadata["InvoiceOperateTiime"] = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");//记录关联发票的操作日期
                payment.AddOnlyProperties(o => o.Metadata);
            }

            return rtns;
        }
    }
}
